#ifndef DPATH
#define DPATH

#include "DEFS.h"

typedef struct paths {
int TimeGridSize;
int RateGridSize;
int Lag;
int PeriodLength;
int StartDate;
int EndDate;
int CurrentTimeIndex;
int RunningTimeIndex;
int CurrentRate;
int **ValidRate;
int *AllValidRates;
REAL **History;
REAL **InitLags;
} PATH;


PATH *CreatePath(int, int, int);
void PATH_Free(PATH *);
int GetCurrentTimeSlice(PATH *, REAL *);
int GetCurrentMCStep(PATH *, REAL *);
int GetPreviousTimeSlice(PATH *, REAL *);
int GetNextTimeSlice(PATH *, REAL *);
void FlipAllHistories(PATH *);
void UpdateCurrentTimeSlice(PATH *, REAL *);
void ShiftCurrentRateHistory(PATH *, REAL );
void ShiftCurrentRate(PATH *, REAL );
int SetCurrentPath(PATH *, REAL * );
void GetCurrentPath(PATH *, REAL * );
void GetInitLagsofCurrentPath(PATH *, REAL * );
void SetInitLagsofCurrentPath(PATH *, REAL * );
int SetStraightPath(PATH *, REAL );
void SetAllPathsToZero(PATH *);
int SetCurrentTimeSlice(PATH *, REAL * );
int SetInitLagsToZero(PATH *);
int SetAllInitLags(PATH *, REAL ** );
int SetInitLag(int, PATH *, REAL *);
void SetTimeGridSize(PATH *, int);
void SetRateGridSize(PATH *, int);
void FetchRunningTimeSlice(PATH *, REAL * );
void PATH_Add(PATH *, PATH *);


#ifdef INLINE

#define ProcessOrder(PathParams) ((PathParams)->Lag)
#define HowManyRates(PathParams) ((PathParams)->RateGridSize)
#define HowManyTimeSteps(PathParams) ((PathParams)->TimeGridSize)
#define SetCurrentTimetoInitial(PathParams) ((PathParams)->CurrentTimeIndex = 0)
#define SetCurrentTimeIndex(PathParams,j)   if(((j)>(PathParams)->TimeGridSize) || ((j)<0)) exit(1);  (PathParams)->CurrentTimeIndex = (j);
#define GetCurrentTimeIndex(PathParams) ((PathParams)->CurrentTimeIndex)
#define SetRunningTimetoCurrent(PathParams) (((PathParams)->RunningTimeIndex) = GetCurrentTimeIndex(PathParams))
#define SetRunningTimeOneStepBack(PathParams) (((PathParams)->RunningTimeIndex) = (((PathParams)->RunningTimeIndex)-1))
#define GetPreviousTimeIndex(PathParams) (((PathParams)->CurrentTimeIndex) - 1)
#define GetNextTimeIndex(PathParams) (((PathParams)->CurrentTimeIndex) + 1)
#define SetCurrentTimetoNext(PathParams) ((PathParams)->CurrentTimeIndex = (PathParams)->CurrentTimeIndex + 1)
#define BeyondHorizon(PathParams) ((GetCurrentTimeIndex(PathParams)) > (HowManyTimeSteps(PathParams)) ? 1: 0)
#define OnHorizon(PathParams) ((GetCurrentTimeIndex(PathParams)) == (HowManyTimeSteps(PathParams)) ? 1: 0)
#define SetCurrentRatetoFirst(PathParams) ((PathParams)->CurrentRate = 1)
#define SetCurrentRateIndex(PathParams, ir) (((PathParams)->CurrentRate) = (ir))
#define GetCurrentRateIndex(PathParams) ((PathParams)->CurrentRate)
#define FinishedLastRate(PathParams) (((PathParams)->CurrentRate) > ((PathParams)->RateGridSize) ? 1 : 0)
#define SetCurrentRatetoNext(PathParams) (((PathParams)->CurrentRate) = ((PathParams)->CurrentRate + 1))

#else

int ProcessOrder( PATH *);
int HowManyRates( PATH *);
int HowManyTimeSteps( PATH *);
void SetCurrentTimetoInitial( PATH *);
void SetCurrentTimeIndex( PATH *, int);
int GetCurrentTimeIndex(PATH *);
void SetRunningTimetoCurrent(PATH *);
void SetRunningTimeOneStepBack(PATH *);
int GetPreviousTimeIndex(PATH *);
int GetNextTimeIndex(PATH *);
void SetCurrentTimetoNext(PATH *);
int BeyondHorizon(PATH *);
int OnHorizon( PATH *);
void SetCurrentRatetoFirst( PATH *);
void SetCurrentRateIndex(PATH *, int );
int FinishedLastRate( PATH *);
int GetCurrentRateIndex( PATH *);
void SetCurrentRatetoNext( PATH *);


#endif

#endif
